#!/usr/bin/python
#
# This thing gets the hardware configuraion out of a system

from socket import gethostname
from socket import gethostbyname

import os
import string

# Some systems don't have the _locale module installed
try:
    import locale
except ImportError:
    locale = None

Hardware = []

# The registration server has several ways of dealing with this hardware
# profile.
# One is to add a 'hardware' member to the registration dictionary when
# calling new_system
# Another is to call registration.add_hw_profile(sysid, Hardware)
#       (accordingly, there is a registration.delete_hw_profile(sysid)

def read_kudzu():
    if not os.access("/etc/sysconfig/hwconf", os.R_OK):
        return
    hwlist = open("/etc/sysconfig/hwconf", "r").read()
    hwlist = string.split(hwlist, "-\n")
    for hw in hwlist:
        if not len(hw):
            continue
        items = string.split(hw, '\n')
        hwdict = {}
        for item in items:
            if not len(item):
                continue
            vals = string.split(item, ":")    
            # Some of the first words are used as dict keys server side
            # so this just helps make that easier
            strippedstring = string.strip(vals[1])
            vals[1] = strippedstring
            hwdict[vals[0]] = string.join(vals[1:])
        Hardware.append(hwdict)

def read_cpuinfo():
    def get_entry(a, entry):
        e = string.lower(entry)
        if not a.has_key(e):
            return ""
        return a[e]

    if not os.access("/proc/cpuinfo", os.R_OK):
        return

    # Okay, the kernel likes to give us the information we need in the
    # standard "C" locale.
    if locale:
        # not really needed if you don;t plan on using atof()
        locale.setlocale(locale.LC_NUMERIC, "C")

    cpulist = open("/proc/cpuinfo", "r").read()
    uname = string.lower(os.uname()[4])

    # This thing should return a hwdict that has the following
    # members:
    #
    # class, desc (required to identify the hardware device)
    # count, type, model, model_number, model_ver, model_rev
    # bogomips, platform, speed, cache
    
    hwdict = { 'class': "CPU",
               "desc" : "Processor",
               }
    if uname[0] == "i" and uname[-2:] == "86":
        # IA32 compatible enough
        count = 0
        tmpdict = {}
        for cpu in string.split(cpulist, "\n\n"):
            if not len(cpu):
                continue
            count = count + 1
            if count > 1:
                continue # just count the rest
            for cpu_attr in string.split(cpu, "\n"):
                if not len(cpu_attr):
                    continue
                vals = string.split(cpu_attr, ":")
                if len(vals) != 2:
                    # XXX: make at least some effort to recover this data...
                    continue
                name, value = string.strip(vals[0]), string.strip(vals[1])
                tmpdict[string.lower(name)] = value
        # XXX: There has to be a better way of doing this.
        hwdict['count']         = count
        hwdict['type']          = get_entry(tmpdict, 'vendor_id')
        hwdict['model']         = get_entry(tmpdict, 'model name')
        hwdict['model_number']  = get_entry(tmpdict, 'cpu family')
        hwdict['model_ver']     = get_entry(tmpdict, 'model')
        hwdict['model_rev']     = get_entry(tmpdict, 'stepping')
        hwdict['cache']         = get_entry(tmpdict, 'cache size')
        hwdict['bogomips']      = get_entry(tmpdict, 'bogomips')
        hwdict['other']         = get_entry(tmpdict, 'flags')
        mhz_speed               = get_entry(tmpdict, 'cpu mhz')
        hwdict['speed']         = int(round(float(mhz_speed)))
        hwdict['platform']      = "i386"

    elif uname == "alpha":
        # Treat it as an an Alpha
        tmpdict = {}
        for cpu_attr in string.split(cpulist, "\n"):
            if not len(cpu_attr):
                continue
            vals = string.split(cpu_attr, ":")
            if len(vals) != 2:
                # XXX: make at least some effort to recover this data...
                continue
            name, value = string.strip(vals[0]), string.strip(vals[1])
            tmpdict[string.lower(name)] = string.lower(value)
        # XXX: There has to be a better way of doing this.
        hwdict['count']         = get_entry(tmpdict, 'cpus detected')
        hwdict['type']          = get_entry(tmpdict, 'cpu')
        hwdict['model']         = get_entry(tmpdict, 'cpu model')
        hwdict['model_number']  = get_entry(tmpdict, 'cpu variation')
        hwdict['model_version'] = "%s/%s" % (get_entry(tmpdict, 'system type'),
                                             get_entry(tmpdict,'system variation'))
        hwdict['model_rev']     = get_entry(tmpdict, 'cpu revision')
        hwdict['cache']         = "" # pitty.
        hwdict['bogomips']      = get_entry(tmpdict, 'bogomips')
        hwdict['other']         = get_entry(tmpdict, 'platform string')
        hz_speed                = get_entry(tmpdict, 'cycle frequency [Hz]')
        hwdict['speed']         = int(round(float(hz_speed))) / 1000000
        hwdict['platform']      = "alpha"

    else:
        # XXX: expand me. Be nice to others
        hwdict['platform']      = uname
        hwdict['count']         = 1 # Good as any
        hwdict['type']          = uname
        hwdict['model']         = uname
        hwdict['model_number']  = ""
        hwdict['model_ver']     = ""
        hwdict['model_rev']     = ""
        hwdict['cache']         = ""
        hwdict['bogomips']      = ""
        hwdict['other']         = ""
        hwdict['speed']         = 0
        
    # This whole things hurts a lot.
    Hardware.append(hwdict)


def read_memory():
    if not os.access("/proc/meminfo", os.R_OK):
        return

    meminfo = open("/proc/meminfo", "r").read()
    lines = string.split(meminfo,"\n")
    curline = lines[1]
    memlist = string.split(curline)
    memdict = {}
    memdict['class'] = "MEMORY"
    megs = string.atoi(memlist[1])/(1024*1024)
    if megs < 32:
        megs = megs + (4 - (megs % 4))
    else:
        megs = megs + (16 - (megs % 16))
    memdict['ram'] = str(megs)
    curline = lines[2]
    memlist = string.split(curline)
    megs = string.atoi(memlist[1])/(1024*1024)
    memdict['swap'] = str(megs)
    Hardware.append(memdict)


def read_network():
    netdict = {}
    netdict['class'] = "NETINFO"
    
    netdict['hostname'] = gethostname()
    netdict['ipaddr'] = gethostbyname(gethostname())
    Hardware.append(netdict)
    
# XXX: Need more functions here:
#  - filesystems layout (/proc.mounts and /proc/mdstat)
#  - is the kudzu config enough or should we strat chasing lscpi and tru to parse that
#    piece of crap output?

#
# Main program
#
if __name__ == '__main__':
    read_kudzu()
    read_cpuinfo()
    read_memory()
    read_network()
    
    for hw in Hardware:
        print hw
        print
