#!/usr/bin/python
#
# RHN Registration Client
# Copyright (c) 2000 Red Hat, Inc.
#
# Author: Preston Brown <pbrown@redhat.com>

import xmlrpclib, cgiwrap
import rpm
import os, sys
import socket
import string

from translate import _, N_, cat

def hasSSL():
    try:
        from httplib import HTTPS
    except ImportError:
        return 0

    return 1

if hasSSL():
    serverUrl = "https://www.rhns.redhat.com/XMLRPC"
else:
    serverUrl = "http://www.rhns.redhat.com/XMLRPC"

class Error:
    # base class for client errors
    def __init__(self, errmsg):
        self.errmsg = errmsg

    def __repr__(self):
        return self.errmsg
    
class FileError(Error):
    def __init__(self, errmsg):
        Error.__init__(self, errmsg)

    def __repr__(self):
        return _("Disk error.  The message was:\n") + self.errmsg

class DelayError(Error):
    def __init__(self, errmsg):
        Error.__init__(self,"Delay requested from server.")
        self.time = atoi(errmsg)

    def __repr__(self):
        return _("Server said come back in %s minutes.") % self.time

    
class CommunicationError(Error):
    def __init__(self, errmsg):
        Error.__init__(self, errmsg)

    # indicates a problem doing xmlrpc communication
    def __repr__(self):
        return _("Error communicating with server.  The message was:\n") + self.errmsg

class ValidationError(Error):
    def __init__(self, errmsg):
        Error.__init__(self, errmsg)

    # indicates an error during server input validation
    def __repr__(self):
        return _("Error validating data at server:\n") + self.errmsg


def validateEmail(email):
    ret = 1

    if len(email) < 6:
        ret = 0
    if not string.find(email, "@") > 0:
        ret = 0
    if not string.find(email, ".") > 0:
        ret = 0

    return ret

def getArch():
    s = os.uname()[4]
    #s = re.sub("i.86", "i386", s)
    return s

def getVersion():
    ret = ""
    db = rpm.opendb(0, "/")
    index = db.findbyname("redhat-release")[0]
    ret = (db[index])['version']
    return ret

# product info structure
productInfo = {
    "regNum" : "",
    "title" : "",
    "first_name" : "",
    "last_name" : "",
    "company" : "",
    "position" : "",
    "address1" : "",
    "address2" : "",
    "city" : "",
    "state" : "",
    "zip" : "",
    "country" : "",
    "phone" : "",
    "fax" : "",
    "contact_email" : 0,
    "contact_mail" : 0,
    "contact_phone" : 0,
    "newsletter" : 0,
    "special_offers" : 0
    }
    
def buildPackageList(callBack = None):
    list = []
    count = 1
    total = 0
    
    db = rpm.opendb(0, "/")
    if "firstkey" in dir(db):
        key = db.firstkey()
        h = db[key]
    else:
        iterator = db.match(0)
        h = iterator.next()
        
    while (h):
        count = count + 1
        if "nextkey" in dir(db):
            key = db.nextkey(key)
            if key:
                h = db[key]
            else:
                h = None
            
        else:
            h = iterator.next()
        
    total = count
    count = 1
    if "firstkey" in dir(db):
        key = db.firstkey()
        h = db[key]
    else:
        iterator = db.match(0)
        h = iterator.next()
    while (h):
        name = h['name']
        epoch = h['epoch']
        if epoch == None:
            epoch = ""
        version = h['version']
        release = h['release']

        list.append([name, version, release, epoch])
        if not callBack == None:
            callBack(count, total)
        count = count + 1
        if "nextkey" in dir(db):
            key = db.nextkey(key)
            if key:
                h = db[key]
            else:
                h = None
        else:
            h = iterator.next()

    list.sort()
    return list
        
def registered():
    return os.access("/etc/sysconfig/rhn/systemid", os.R_OK)

def writeSystemId(systemId):
    if not os.access("/etc/sysconfig/rhn", os.W_OK):
        try:
            os.mkdir("/etc/sysconfig/rhn")
        except:
            return 0
    if not os.access("/etc/sysconfig/rhn", os.W_OK):
        return 0

    f = open("/etc/sysconfig/rhn/systemid", "w")
    f.write(systemId)
    f.close()
    try:
        os.chmod("/etc/sysconfig/rhn/systemid", 0600)
    except:
        return 0
    return 1


def welcomeText():
    s = xmlrpclib.Server(serverUrl)
    if os.environ.has_key('LANG'):
        s.setlang(os.environ['LANG'])

    try:
        return s.registration.welcome_message()
    except socket.error, e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        if f.faultCode == 99:
            raise DelayError(f.faultString)
        else:
            raise CommunicationError(f.faultString)
    

def privacyText():
    s = xmlrpclib.Server(serverUrl)
    if os.environ.has_key('LANG'):
        s.setlang(os.environ['LANG'])

    try:
        return s.registration.privacy_statement()
    except socket.error, e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        if f.faultCode == 99:
            raise DelayError(f.faultString)
        else:
            raise CommunicationError(f.faultString)
    

def reserveUser(username, password):
    s = xmlrpclib.Server(serverUrl)
    if os.environ.has_key('LANG'):
        s.setlang(os.environ['LANG'])

    try:
        ret = s.registration.reserve_user(username, password)
    except socket.error, e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        if f.faultCode == -3:
            # account already in use
            raise ValidationError(f.faultString)
        elif f.faultCode == -14:
            # too short password
            raise ValidationError(f.faultString)
        elif f.faultCode == -15:
            # bad chars in username
            raise ValidationError(f.faultString)
        elif f.faultCode == 99:
            raise DelayError(f.faultString)
        else:
            raise CommunicationError(f.faultString)

    if ret == 0:
        return 1
    else:
        return 0

def validateRegNum(regNum):
    s = xmlrpclib.Server(serverUrl)
    if os.environ.has_key('LANG'):
        s.setlang(os.environ['LANG'])

    try:
        s.registration.validate_reg_num(regNum)
    except socket.error, e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        if f.faultCode == -16:
            # invalid
            raise ValidationError(f.faultString)
        elif f.faultCode == 99:
            raise DelayError(f.faultString)
        else:
            raise CommunicationError(f.faultString)


def registerUser(username, password, email = None):
    s  = xmlrpclib.Server(serverUrl)
    if os.environ.has_key('LANG'):
        s.setlang(os.environ['LANG'])

    try:
        if not email == None:
            s.registration.new_user(username, password, email)
        else:
            s.registration.new_user(username, password)
    except socket.error, e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        if f.faultCode == 99:
            raise DelayError(f.faultString)
        else:
            raise CommunicationError(f.faultString)


def registerSystem(username, password,
                   profileName, packages = None):
    s = xmlrpclib.Server(serverUrl)
    if os.environ.has_key('LANG'):
        s.setlang(os.environ['LANG'])

    try:
        if packages == None:
            ret = s.registration.new_system({ "username": username,
                                              "password": password,
                                              "profile_name" : profileName,
                                              "os_release" : getVersion(),
                                              "architecture" : getArch() })
        else:
            ret = s.registration.new_system({ "username": username,
                                              "password": password,
                                              "profile_name" : profileName,
                                              "os_release" : getVersion(),
                                              "architecture" : getArch() },
                                            packages)
    except socket.error, e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        if f.faultCode == 99:
            raise DelayError(f.faultString)
        else:
            raise CommunicationError(f.faultString)

    return ret


def registerProduct(systemId, productInfo):
    s = xmlrpclib.Server(serverUrl)
    if os.environ.has_key('LANG'):
        s.setlang(os.environ['LANG'])

    try:
        s.registration.register_product(systemId, productInfo)
    except socket.error, e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        if f.faultCode == 99:
            raise DelayError(f.faultString)
        else:
            raise CommunicationError(f.faultString)


def sendHardware(systemId, hardwareList):
    s = xmlrpclib.Server(serverUrl)
    if os.environ.has_key('LANG'):
        s.setlang(os.environ['LANG'])

    try:
        s.registration.add_hw_profile(systemId, hardwareList)
    except socket.error, e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        print f
        if f.faultCode == 99:
            raise DelayError(f.faultString)
        else:
            raise CommunicationError(f.faultString)

def sendPackages(systemId, packageList):
    s = xmlrpclib.Server(serverUrl)
    if os.environ.has_key('LANG'):
        s.setlang(os.environ['LANG'])

    try:
        s.registration.add_packages(systemId, packageList)
    except socket.error, e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        if f.faultCode == 99:
            raise DelayError(f.faultString)
        else:
            raise CommunicationError(f.faultString)

def listPackages(systemId):
    s = xmlrpclib.Server(serverUrl)
    if os.environ.has_key('LANG'):
        s.setlang(os.environ['LANG'])

    try:
        print s.registration.list_packages(systemId)
    except socket.error, e:
        if len(e.args) > 1:
            raise CommunicationError(e.args[1])
        else:
            raise CommunicationError(e.args[0])
    except cgiwrap.ProtocolError, e:
        raise CommunicationError(e.errmsg)
    except xmlrpclib.Fault, f:
        if f.faultCode == 99:
            raise DelayError(f.faultString)
        else:
            raise CommunicationError(f.faultString)


def updatePackages(systemId):
    s = xmlrpclib.Server(serverUrl)
    if os.environ.has_key('LANG'):
        s.setlang(os.environ['LANG'])
        
    try:
        s.registration.add_packages(systemId, buildPackageList())
    except xmlrpclib.Fault, f:
        if f.faultCode == 99:
            raise DelayError(f.faultString)
        else:
            raise CommunicationError(f.faultString)
